import { IComputedValue } from 'mobx/dist/internal';
import { RouterStore } from 'mobx-react-router';

declare enum FalsyValue {
    'EmptyString' = 0,
    'False' = 1,
    'NaN' = 2,
    'Null' = 3,
    'Undefined' = 4,
    'Zero' = 5
}
type FormatFunction<T> = (it: T | null | undefined) => string;
type ComputedRouteOptions<T = string> = {
    patterns: string[];
    parse?: (it: string) => T;
    format?: FormatFunction<T>;
    defaultValue?: T;
    keepAlive?: boolean;
};
type SetRouteParamOptions = {
    enforce?: boolean;
    enforcePattern?: string | undefined | null | false;
    cleanParams?: boolean | ComputedRouteParam<any>[];
    falsyValuesAllowed?: FalsyValue[];
};
type ComputedRouteParam<T> = IComputedValue<T> & {
    push: RouteParamSetter<T>;
    replace: RouteParamSetter<T>;
    __name: string;
};
type RouteParamSetter<T> = (newValue: T | null | undefined, options?: SetRouteParamOptions) => Promise<string> | void;
type TypeComputedRouteOptions<T> = Omit<ComputedRouteOptions<T>, 'format' | 'parse'>;

declare const setRoutingStore: (routingStore: RouterStore) => RouterStore;

declare const computedRouteParam: {
    <T = string>(paramName: string, { patterns: routePatterns, parse: _parse, format, defaultValue, keepAlive }: ComputedRouteOptions<T>): ComputedRouteParam<T>;
    date(paramName: string, options: TypeComputedRouteOptions<Date>): ComputedRouteParam<Date>;
    int(paramName: string, options: TypeComputedRouteOptions<number>): ComputedRouteParam<number>;
    float(paramName: string, options: TypeComputedRouteOptions<number>): ComputedRouteParam<number>;
    boolean(paramName: string, options: TypeComputedRouteOptions<boolean>): ComputedRouteParam<boolean>;
};

export { computedRouteParam, setRoutingStore };
